# Generated by Django 1.11.15 on 2018-08-06 21:58


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0010_enterpriseenrollment_created'),
    ]

    operations = [
        migrations.CreateModel(
            name='EnterpriseUser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('enterprise_id', models.UUIDField()),
                ('lms_user_id', models.PositiveIntegerField()),
                ('enterprise_user_id', models.PositiveIntegerField(unique=True)),
                ('enterprise_sso_uid', models.CharField(max_length=255, null=True)),
                ('user_account_creation_timestamp', models.DateTimeField(null=True)),
                ('user_email', models.CharField(max_length=255, null=True)),
                ('user_username', models.CharField(max_length=255, null=True)),
                ('user_country_code', models.CharField(max_length=2, null=True)),
                ('last_activity_date', models.DateField(null=True)),
            ],
            options={
                'db_table': 'enterprise_user',
                'verbose_name': 'Enterprise User',
                'verbose_name_plural': 'Enterprise Users',
            },
        ),
    ]



# Generated by Django 3.2.19 on 2023-06-09 13:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0029_enterpriseoffer'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterpriselearnerenrollment',
            name='offer_id',
            field=models.CharField(max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='enterpriseoffer',
            name='offer_id',
            field=models.CharField(max_length=255, primary_key=True, serialize=False),
        ),
    ]



# Generated by Django 1.11.15 on 2018-09-07 17:57


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0014_enterpriseuser_created'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='enterpriseuser',
            options={'ordering': ['-user_email'], 'verbose_name': 'Enterprise User', 'verbose_name_plural': 'Enterprise Users'},
        ),
    ]



"""
URL definitions for enterprise data API endpoint.
"""


from django.urls import include, path

urlpatterns = [
    path('v0/', include('enterprise_data.api.v0.urls', 'v0'), name='api_v0'),
    path('v1/', include('enterprise_data.api.v1.urls', 'v1'), name='api_v1'),
]



# Generated by Django 1.11.15 on 2018-08-31 19:30


import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0011_enterpriseuser'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterpriseenrollment',
            name='enterprise_user_id',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='enrollments', to='enterprise_data.EnterpriseUser', to_field='enterprise_user_id'),
        ),
        migrations.AlterField(
            model_name='enterpriseuser',
            name='enterprise_user_id',
            field=models.PositiveIntegerField(unique=True),
        ),
    ]



# pylint: skip-file
"""
Enterprise Data Django application initialization.
"""

from django.apps import AppConfig


class EnterpriseDataAppConfig(AppConfig):
    """
    Configuration for the enterprise Django application.
    """

    name = "enterprise_data"

    def ready(self) -> None:
        """
        Perform tasks when the Django application is ready.
        """
        import enterprise_data.signals



"""
Test cases for enterprise_admin views
"""
from datetime import datetime
from unittest import mock

import ddt
from mock import patch
from pytest import mark
from rest_framework import status
from rest_framework.reverse import reverse
from rest_framework.test import APITransactionTestCase

from enterprise_data.admin_analytics.database.queries import (
    FactEngagementAdminDashQueries,
    FactEnrollmentAdminDashQueries,
)
from enterprise_data.tests.admin_analytics.mock_analytics_data import (
    TOP_SKILLS,
    TOP_SKILLS_BY_COMPLETIONS,
    TOP_SKILLS_BY_ENROLLMENTS,
)
from enterprise_data.tests.mixins import JWTTestMixin
from enterprise_data.tests.test_utils import EnterpriseSubsidyBudgetFactory, UserFactory, get_dummy_enterprise_api_data
from enterprise_data_roles.constants import ENTERPRISE_DATA_ADMIN_ROLE
from enterprise_data_roles.models import EnterpriseDataFeatureRole, EnterpriseDataRoleAssignment


@ddt.ddt
@mark.django_db
class TestEnterpriseAdminAnalyticsAggregatesView(JWTTestMixin, APITransactionTestCase):
    """
    Tests for EnterpriseAdminAnalyticsAggregatesView.
    """

    def setUp(self):
        """
        Setup method.
        """
        super().setUp()
        self.user = UserFactory(is_staff=True)
        role, __ = EnterpriseDataFeatureRole.objects.get_or_create(name=ENTERPRISE_DATA_ADMIN_ROLE)
        self.role_assignment = EnterpriseDataRoleAssignment.objects.create(
            role=role,
            user=self.user
        )
        self.client.force_authenticate(user=self.user)

        mocked_get_enterprise_customer = mock.patch(
            'enterprise_data.filters.EnterpriseApiClient.get_enterprise_customer',
            return_value=get_dummy_enterprise_api_data()
        )

        self.mocked_get_enterprise_customer = mocked_get_enterprise_customer.start()
        self.addCleanup(mocked_get_enterprise_customer.stop)
        self.enterprise_id = 'ee5e6b3a-069a-4947-bb8d-d2dbc323396c'
        self.set_jwt_cookie()
        self.enrollment_queries = FactEnrollmentAdminDashQueries()
        self.engagement_queries = FactEngagementAdminDashQueries()

    def _mock_run_query(self, query, *args, **kwargs):
        """
        mock implementation of run_query.
        """
        mock_responses = {
            self.enrollment_queries.get_enrollment_date_range_query(): [[
                datetime.strptime('2021-01-01', "%Y-%m-%d"),
                datetime.strptime('2021-12-31', "%Y-%m-%d"),
            ]],
            self.enrollment_queries.get_enrollment_and_course_count_query(): [[
                100, 10
            ]],
            self.enrollment_queries.get_completion_count_query(): [[
                50
            ]],
            self.engagement_queries.get_learning_hours_and_daily_sessions_query(): [[
                100, 10
            ]],
            'SELECT MAX(created) FROM enterprise_learner_enrollment': [[datetime.strptime('2021-01-01', "%Y-%m-%d")]]
        }
        return mock_responses[query]

    def test_get_admin_analytics_aggregates(self):
        """
        Test to get admin analytics aggregates.
        """
        url = reverse('v1:enterprise-admin-analytics-aggregates', kwargs={'enterprise_id': self.enterprise_id})
        with patch('enterprise_data.admin_analytics.data_loaders.run_query', side_effect=self._mock_run_query):
            with patch(
                    'enterprise_data.admin_analytics.database.tables.fact_engagement_admin_dash.run_query',
                    side_effect=self._mock_run_query
            ):
                with patch(
                        'enterprise_data.admin_analytics.database.tables.fact_enrollment_admin_dash.run_query',
                        side_effect=self._mock_run_query
                ):
                    response = self.client.get(url)
                    assert response.status_code == status.HTTP_200_OK
                    assert 'enrolls' in response.json()
                    assert 'courses' in response.json()
                    assert 'completions' in response.json()
                    assert 'hours' in response.json()
                    assert 'sessions' in response.json()
                    assert 'last_updated_at' in response.json()
                    assert 'min_enrollment_date' in response.json()
                    assert 'max_enrollment_date' in response.json()


@ddt.ddt
class TestSkillsStatsAPI(JWTTestMixin, APITransactionTestCase):
    """Tests for EnterpriseAdminAnalyticsSkillsView."""

    def setUp(self):
        """
        Setup method.
        """
        super().setUp()
        self.user = UserFactory(is_staff=True)
        role, __ = EnterpriseDataFeatureRole.objects.get_or_create(
            name=ENTERPRISE_DATA_ADMIN_ROLE
        )
        self.role_assignment = EnterpriseDataRoleAssignment.objects.create(
            role=role, user=self.user
        )
        self.client.force_authenticate(user=self.user)

        self.enterprise_uuid = "ee5e6b3a069a4947bb8dd2dbc323396c"
        self.set_jwt_cookie()

        self.url = reverse(
            "v1:enterprise-admin-analytics-skills",
            kwargs={"enterprise_id": self.enterprise_uuid},
        )

    @patch('enterprise_data.api.v1.views.enterprise_admin.FactEnrollmentAdminDashTable.get_enrollment_date_range')
    @patch('enterprise_data.api.v1.views.enterprise_admin.SkillsDailyRollupAdminDashTable.get_top_skills')
    @patch('enterprise_data.api.v1.views.enterprise_admin.SkillsDailyRollupAdminDashTable.get_top_skills_by_enrollment')
    @patch('enterprise_data.api.v1.views.enterprise_admin.SkillsDailyRollupAdminDashTable.get_top_skills_by_completion')
    def test_get(
        self,
        mock_get_top_skills_by_completion,
        mock_get_top_skills_by_enrollment,
        mock_get_top_skills,
        mock_get_enrollment_date_range
    ):
        """
        Test the GET method for the EnterpriseAdminAnalyticsSkillsView works.
        """
        mock_get_enrollment_date_range.return_value = ("2020-04-03", "2024-07-04")
        mock_get_top_skills.return_value = TOP_SKILLS
        mock_get_top_skills_by_enrollment.return_value = TOP_SKILLS_BY_ENROLLMENTS
        mock_get_top_skills_by_completion.return_value = TOP_SKILLS_BY_COMPLETIONS

        response = self.client.get(self.url)
        assert response.status_code == status.HTTP_200_OK
        data = response.json()
        assert data == {
            "top_skills": [
                {
                    "skill_name": "Python (Programming Language)",
                    "skill_type": "Specialized Skill",
                    "enrolls": 19027.0,
                    "completions": 3004.0,
                },
                {
                    "skill_name": "Data Science",
                    "skill_type": "Specialized Skill",
                    "enrolls": 13756.0,
                    "completions": 1517.0,
                },
                {
                    "skill_name": "Algorithms",
                    "skill_type": "Specialized Skill",
                    "enrolls": 12776.0,
                    "completions": 1640.0,
                },
            ],
            "top_skills_by_enrollments": [
                {
                    "skill_name": "Python (Programming Language)",
                    "subject_name": "business-management",
                    "count": 313.0,
                },
                {
                    "skill_name": "Machine Learning",
                    "subject_name": "business-management",
                    "count": 442.0,
                },
                {
                    "skill_name": "Computer Science",
                    "subject_name": "business-management",
                    "count": 39.0,
                },
            ],
            "top_skills_by_completions": [
                {
                    "skill_name": "Python (Programming Language)",
                    "subject_name": "business-management",
                    "count": 21.0,
                },
                {
                    "skill_name": "SQL (Programming Language)",
                    "subject_name": "business-management",
                    "count": 11.0,
                },
                {
                    "skill_name": "Algorithms",
                    "subject_name": "business-management",
                    "count": 15.0,
                },
            ],
        }

    @ddt.data(
        {
            "params": {"start_date": 1},
            "error": {
                "start_date": [
                    "Date has wrong format. Use one of these formats instead: YYYY-MM-DD."
                ]
            },
        },
        {
            "params": {"end_date": 2},
            "error": {
                "end_date": [
                    "Date has wrong format. Use one of these formats instead: YYYY-MM-DD."
                ]
            },
        },
        {
            "params": {"start_date": "2024-01-01", "end_date": "2023-01-01"},
            "error": {
                "non_field_errors": [
                    "start_date should be less than or equal to end_date."
                ]
            },
        },
    )
    @ddt.unpack
    def test_get_invalid_query_params(self, params, error):
        """
        Test the GET method return correct error if any query param value is incorrect.
        """
        response = self.client.get(self.url, params)
        assert response.status_code == status.HTTP_400_BAD_REQUEST
        assert response.json() == error


@ddt.ddt
class TestEnterpriseBudgetAPI(JWTTestMixin, APITransactionTestCase):
    """Tests for EnterpriseBudgetView."""

    def setUp(self):
        """
        Setup method.
        """
        super().setUp()
        self.user = UserFactory(is_staff=True)
        role, __ = EnterpriseDataFeatureRole.objects.get_or_create(
            name=ENTERPRISE_DATA_ADMIN_ROLE
        )
        self.role_assignment = EnterpriseDataRoleAssignment.objects.create(
            role=role, user=self.user
        )
        self.client.force_authenticate(user=self.user)

        self.enterprise_uuid = "ee5e6b3a069a4947bb8dd2dbc323396c"
        self.set_jwt_cookie()

        self.url = reverse(
            "v1:enterprise-budgets",
            kwargs={"enterprise_uuid": self.enterprise_uuid},
        )

        self.enterprise_subsidy_budget = EnterpriseSubsidyBudgetFactory(
            enterprise_customer_uuid=self.enterprise_uuid,
            subsidy_access_policy_uuid='8d6503dd-e40d-42b8-442b-37dd4c5450e3',
            subsidy_access_policy_display_name='test-budget'
        )

    def test_get(self):
        """
        Test the GET method for the EnterpriseBudgetView works.
        """
        response = self.client.get(self.url)
        assert response.status_code == status.HTTP_200_OK
        assert response.json() == [
            {
                'subsidy_access_policy_uuid': '8d6503dd-e40d-42b8-442b-37dd4c5450e3',
                'subsidy_access_policy_display_name': 'test-budget',
            }
        ]



"""Unittests for analytics_enrollments.py"""

from datetime import datetime

import ddt
from mock import patch
from rest_framework import status
from rest_framework.reverse import reverse
from rest_framework.test import APITransactionTestCase

from enterprise_data.admin_analytics.constants import ResponseType
from enterprise_data.tests.admin_analytics.mock_analytics_data import ENGAGEMENTS
from enterprise_data.tests.mixins import JWTTestMixin
from enterprise_data.tests.test_utils import UserFactory
from enterprise_data_roles.constants import ENTERPRISE_DATA_ADMIN_ROLE
from enterprise_data_roles.models import EnterpriseDataFeatureRole, EnterpriseDataRoleAssignment


@ddt.ddt
class TestIndividualEngagementsAPI(JWTTestMixin, APITransactionTestCase):
    """Tests for AdvanceAnalyticsIndividualEngagementsView."""

    def setUp(self):
        """
        Setup method.
        """
        super().setUp()
        self.user = UserFactory(is_staff=True)
        role, __ = EnterpriseDataFeatureRole.objects.get_or_create(
            name=ENTERPRISE_DATA_ADMIN_ROLE
        )
        self.role_assignment = EnterpriseDataRoleAssignment.objects.create(
            role=role, user=self.user
        )
        self.client.force_authenticate(user=self.user)

        self.enterprise_uuid = 'ee5e6b3a-069a-4947-bb8d-d2dbc323396c'
        self.set_jwt_cookie()

        self.url = reverse(
            'v1:enterprise-admin-analytics-engagements',
            kwargs={"enterprise_uuid": self.enterprise_uuid},
        )

        get_enrollment_date_range_patcher = patch(
            'enterprise_data.api.v1.views.analytics_enrollments.FactEnrollmentAdminDashTable.get_enrollment_date_range',
            return_value=(datetime.now(), datetime.now())
        )

        get_enrollment_date_range_patcher.start()
        self.addCleanup(get_enrollment_date_range_patcher.stop)

    @patch('enterprise_data.api.v1.views.analytics_engagements.FactEngagementAdminDashTable.get_engagement_count')
    @patch('enterprise_data.api.v1.views.analytics_engagements.FactEngagementAdminDashTable.get_all_engagements')
    def test_get(self, mock_get_all_engagements, mock_get_engagement_count):
        """
        Test the GET method for the AdvanceAnalyticsIndividualEngagementsView works.
        """
        mock_get_all_engagements.return_value = ENGAGEMENTS
        mock_get_engagement_count.return_value = len(ENGAGEMENTS)

        response = self.client.get(self.url + '?page_size=2')
        assert response.status_code == status.HTTP_200_OK
        data = response.json()
        assert data['next'] == f"http://testserver{self.url}?page=2&page_size=2"
        assert data['previous'] is None
        assert data['current_page'] == 1
        assert data['num_pages'] == 6
        assert data['count'] == 12

        response = self.client.get(self.url + '?page_size=2&page=2')
        assert response.status_code == status.HTTP_200_OK
        data = response.json()
        assert data['next'] == f"http://testserver{self.url}?page=3&page_size=2"
        assert data['previous'] == f"http://testserver{self.url}?page_size=2"
        assert data['current_page'] == 2
        assert data['num_pages'] == 6
        assert data['count'] == 12

        response = self.client.get(self.url + '?page_size=2&page=6')
        assert response.status_code == status.HTTP_200_OK
        data = response.json()
        assert data['next'] is None
        assert data['previous'] == f"http://testserver{self.url}?page=5&page_size=2"
        assert data['current_page'] == 6
        assert data['num_pages'] == 6
        assert data['count'] == 12

        response = self.client.get(self.url + '?page_size=12')
        assert response.status_code == status.HTTP_200_OK
        data = response.json()
        assert data['next'] is None
        assert data['previous'] is None
        assert data['current_page'] == 1
        assert data['num_pages'] == 1
        assert data['count'] == 12

    @patch('enterprise_data.api.v1.views.analytics_engagements.FactEngagementAdminDashTable.get_engagement_count')
    @patch('enterprise_data.api.v1.views.analytics_engagements.FactEngagementAdminDashTable.get_all_engagements')
    def test_get_csv(self, mock_get_all_engagements, mock_get_engagement_count):
        """
        Test the GET method for the AdvanceAnalyticsIndividualEngagementsView return correct CSV data.
        """
        mock_get_all_engagements.return_value = ENGAGEMENTS[0:4]
        mock_get_engagement_count.return_value = 4

        response = self.client.get(self.url, {"response_type": ResponseType.CSV.value})
        assert response.status_code == status.HTTP_200_OK

        # verify the response headers
        assert response["Content-Type"] == "text/csv"

        # verify the response content
        content = b"".join(response.streaming_content).decode().splitlines()
        assert len(content) == 5

        # Verify CSV header.
        assert 'email,course_title,course_subject,enroll_type,activity_date,learning_time_hours' == content[0]

        # verify the content
        assert (
                'padillamichelle@example.org,Synergized reciprocal encoding,business-management,certificate,2021-08-05,'
                '1.0344444444444445'
                in content
        )
        assert (
                'yallison@example.org,Synergized reciprocal encoding,business-management,certificate,2021-07-27,'
                '1.2041666666666666'
                in content
        )
        assert (
                'weaverpatricia@example.net,Synergized reciprocal encoding,business-management,certificate,2021-08-05,'
                '2.6225'
                in content
        )
        assert (
                'seth57@example.org,Synergized reciprocal encoding,business-management,certificate,2021-08-21,'
                '2.7494444444444444'
                in content
        )

    @ddt.data(
        {
            'params': {'start_date': 1},
            'error': {
                'start_date': [
                    'Date has wrong format. Use one of these formats instead: YYYY-MM-DD.'
                ]
            },
        },
        {
            'params': {'end_date': 2},
            'error': {
                'end_date': [
                    'Date has wrong format. Use one of these formats instead: YYYY-MM-DD.'
                ]
            },
        },
        {
            'params': {"start_date": "2024-01-01", "end_date": "2023-01-01"},
            'error': {
                'non_field_errors': [
                    'start_date should be less than or equal to end_date.'
                ]
            },
        },
    )
    @ddt.unpack
    def test_get_invalid_query_params(self, params, error):
        """
        Test the GET method return correct error if any query param value is incorrect.
        """
        response = self.client.get(self.url, params)
        assert response.status_code == status.HTTP_400_BAD_REQUEST
        assert response.json() == error


@ddt.ddt
class TestEngagementStatsAPI(JWTTestMixin, APITransactionTestCase):
    """Tests for AdvanceAnalyticsEngagementStatsView."""

    def setUp(self):
        """
        Setup method.
        """
        super().setUp()
        self.user = UserFactory(is_staff=True)
        role, __ = EnterpriseDataFeatureRole.objects.get_or_create(
            name=ENTERPRISE_DATA_ADMIN_ROLE
        )
        self.role_assignment = EnterpriseDataRoleAssignment.objects.create(
            role=role, user=self.user
        )
        self.client.force_authenticate(user=self.user)

        self.enterprise_uuid = 'ee5e6b3a-069a-4947-bb8d-d2dbc323396c'
        self.set_jwt_cookie()

        self.url = reverse(
            'v1:enterprise-admin-analytics-engagements-stats',
            kwargs={'enterprise_uuid': self.enterprise_uuid},
        )

        get_enrollment_date_range_patcher = patch(
            'enterprise_data.api.v1.views.analytics_enrollments.FactEnrollmentAdminDashTable.get_enrollment_date_range',
            return_value=(datetime.now(), datetime.now())
        )

        get_enrollment_date_range_patcher.start()
        self.addCleanup(get_enrollment_date_range_patcher.stop)

    @patch(
        'enterprise_data.api.v1.views.analytics_engagements.FactEngagementAdminDashTable.'
        'get_engagement_time_series_data'
    )
    @patch(
        'enterprise_data.api.v1.views.analytics_engagements.FactEngagementAdminDashTable.get_top_courses_by_engagement'
    )
    @patch(
        'enterprise_data.api.v1.views.analytics_engagements.FactEngagementAdminDashTable.get_top_subjects_by_engagement'
    )
    def test_get(self, mock_get_top_subjects_by_engagement, mock_get_top_courses_by_engagement, mock_get_time_series):
        """
        Test the GET method for the AdvanceAnalyticsEnrollmentStatsView works.
        """
        mock_get_top_subjects_by_engagement.return_value = []
        mock_get_top_courses_by_engagement.return_value = []
        mock_get_time_series.return_value = []

        response = self.client.get(self.url)
        assert response.status_code == status.HTTP_200_OK
        data = response.json()
        assert 'engagement_over_time' in data
        assert 'top_courses_by_engagement' in data
        assert 'top_subjects_by_engagement' in data

    @ddt.data(
        {
            'params': {'start_date': 1},
            'error': {
                'start_date': [
                    'Date has wrong format. Use one of these formats instead: YYYY-MM-DD.'
                ]
            },
        },
        {
            'params': {'end_date': 2},
            'error': {
                'end_date': [
                    'Date has wrong format. Use one of these formats instead: YYYY-MM-DD.'
                ]
            },
        },
        {
            'params': {'start_date': '2024-01-01', 'end_date': '2023-01-01'},
            'error': {
                'non_field_errors': [
                    'start_date should be less than or equal to end_date.'
                ]
            },
        },
    )
    @ddt.unpack
    def test_get_invalid_query_params(self, params, error):
        """
        Test the GET method return correct error if any query param value is incorrect.
        """
        response = self.client.get(self.url, params)
        assert response.status_code == status.HTTP_400_BAD_REQUEST
        assert response.json() == error



# Generated by Django 4.2.15 on 2024-12-23 12:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0044_enterpriseexecedlcmoduleperformance'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='enterpriseexecedlcmoduleperformance',
            options={'verbose_name': 'Exec Ed LC Module Performance', 'verbose_name_plural': 'Exec Ed LC Module Performance'},
        ),
        migrations.AddField(
            model_name='enterpriselearnerenrollment',
            name='user_first_name',
            field=models.CharField(max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='enterpriselearnerenrollment',
            name='user_last_name',
            field=models.CharField(max_length=255, null=True),
        ),
    ]



"""
Classes that handle sending reports for EnterpriseCustomers.
"""

import csv
import datetime
import json
import logging
import os
from collections import OrderedDict
from uuid import UUID

from enterprise_reporting.clients.enterprise import (
    AnalyticsDataApiClient,
    EnterpriseAPIClient,
    EnterpriseCatalogAPIClient,
    EnterpriseDataApiClient,
    EnterpriseDataV1ApiClient,
)
from enterprise_reporting.clients.s3 import S3Client
from enterprise_reporting.clients.vertica import VerticaClient
from enterprise_reporting.delivery_method import SFTPDeliveryMethod, SMTPDeliveryMethod
from enterprise_reporting.utils import (
    decrypt_string,
    extract_catalog_uuids_from_reporting_config,
    generate_data,
    retry_on_exception,
)

LOGGER = logging.getLogger(__name__)
NOW = datetime.datetime.now().strftime("%Y-%m-%d")


class EnterpriseReportSender:
    """
    Class that handles the process of sending a data report to an Enterprise Customer.
    """

    VERTICA_QUERY = ("SELECT {fields} FROM business_intelligence.enterprise_enrollment"
                     " WHERE enterprise_id = '{enterprise_id}' AND consent_granted = 1")
    VERTICA_QUERY_FIELDS = (
        'enterprise_user_id',
        'lms_user_id',
        'enterprise_sso_uid',
        'enrollment_created_timestamp',
        'consent_granted',
        'course_id',
        'course_title',
        'course_duration',
        'course_min_effort',
        'course_max_effort',
        'user_account_creation_date',
        'user_email',
        'user_username',
        'user_age',
        'user_level_of_education',
        'user_gender',
        'user_country_code',
        'country_name',
        'has_passed',
        'passed_timestamp',
        'time_spent_hours',
        'last_activity_date',
        'user_current_enrollment_mode',
    )

    FILE_WRITE_DIRECTORY = '/tmp'

    def __init__(self, reporting_config, delivery_method):
        """Initialize with an EnterpriseCustomerReportingConfiguration."""
        self.reporting_config = reporting_config
        self.delivery_method = delivery_method
        self.enterprise_customer_uuid = reporting_config['enterprise_customer']['uuid']
        self.enterprise_customer_name = reporting_config['enterprise_customer']['name']
        self.data_type = reporting_config['data_type']
        self.report_type = reporting_config['report_type']

    @staticmethod
    def create(reporting_config):
        """Create the EnterpriseReportSender and all of its dependencies."""
        enterprise_customer_name = reporting_config['enterprise_customer']['name']
        delivery_method_str = reporting_config['delivery_method']
        if delivery_method_str == 'email':
            LOGGER.debug('{} is configured to send the report via SMTP to {}'.format(
                enterprise_customer_name,
                reporting_config['email'],
            ))
            decrypted_password = None
            if reporting_config['encrypted_password']:
                decrypted_password = decrypt_string(reporting_config['encrypted_password'])
            delivery_method = SMTPDeliveryMethod(reporting_config, decrypted_password)
        elif delivery_method_str == 'sftp':
            LOGGER.debug('{} is configured to send the report via SFTP to {}'.format(
                enterprise_customer_name,
                reporting_config['sftp_hostname'],
            ))
            delivery_method = SFTPDeliveryMethod(
                reporting_config,
                decrypt_string(reporting_config['encrypted_sftp_password']),
            )
        else:
            raise ValueError(f'Invalid delivery method: {delivery_method_str}')

        return EnterpriseReportSender(reporting_config, delivery_method)

    @property
    def data_report_file_name(self):
        """Get the full path to the report file."""
        date_str = f"_{NOW}" if self.reporting_config.get('include_date') else ""
        return "{dir}/{enterprise_id}_{data}_{ext}{date_str}.{ext}".format(
            dir=self.FILE_WRITE_DIRECTORY,
            enterprise_id=self.enterprise_customer_uuid,
            data=self.data_type,
            date_str=date_str,
            ext=self.report_type,
        )

    @property
    def data_report_file_name_with(self):
        """Get a full path to the report file that can be modified with arbitrary formatting."""
        return '_{}.'.join(self.data_report_file_name.rsplit('.'))

    @retry_on_exception(max_retries=2, delay=1, backoff=2)
    def send_enterprise_report(self):
        """Generate the report file of the appropriate type and send it through the configured delivery method."""
        LOGGER.info(f'Starting process to send report to {self.enterprise_customer_name}')
        files = self._generate_enterprise_report()
        if files:
            self.delivery_method.send(files)
        else:
            LOGGER.warning('No {} {} reports were generated for {}! Moving on...'.format(
                self.data_type,
                self.report_type,
                self.enterprise_customer_name
            ))

    def _generate_enterprise_report(self):
        """Calls the appropriate method for generating the report, e.g. the method for a CSV report of Catalog data."""
        LOGGER.info(f'Generating {self.data_type} report in {self.report_type} format...')
        return getattr(self, '_generate_enterprise_report_{type}_{ext}'.format(
            type=self.data_type,
            ext=self.report_type
        ))()

    def _generate_enterprise_report_progress_csv(self):
        """Query vertica and write output to csv file."""
        vertica_client = VerticaClient()
        vertica_client.connect()
        with open(self.data_report_file_name, 'w') as data_report_file:
            data_report_file_writer = csv.writer(data_report_file)
            data_report_file_writer.writerow(self.VERTICA_QUERY_FIELDS)
            query = self.VERTICA_QUERY.format(
                fields=','.join(self.VERTICA_QUERY_FIELDS),
                enterprise_id=UUID(self.enterprise_customer_uuid).hex
            )
            LOGGER.debug(f'Executing this Vertica query: {query}')
            data_report_file_writer.writerows(vertica_client.stream_results(query))
        vertica_client.close_connection()
        return [data_report_file]

    def get_s3_csv_path(self):
        """Return S3 CSV path for a manual report else raise an exception if path doesn't exist"""
        # <data_type>-<enterprise uuid>
        env_variable_name = f"{self.data_type}-{self.enterprise_customer_uuid.replace('-', '')}"
        s3_csv_path = os.environ.get(env_variable_name)

        if not s3_csv_path:
            raise ValueError(f'Invalid S3 CSV path. Path: [{s3_csv_path}]')

        LOGGER.info(
            'Enterprise UUID: [%s], Enterprise Name: [%s], Data Type: [%s], S3 CSV Path: [%s]',
            self.enterprise_customer_uuid,
            self.enterprise_customer_name,
            self.data_type,
            s3_csv_path,
        )

        return s3_csv_path

    def _generate_enterprise_report_grade_csv(self):
        """Query S3 and write output to csv file."""
        s3_client = S3Client()
        with open(self.data_report_file_name, 'wb') as data_report_file:
            LOGGER.debug('Fetching enterprise grade report from S3')
            s3_client.get_enterprise_report(self.get_s3_csv_path(), data_report_file)
        return [data_report_file]

    def _generate_enterprise_report_course_structure_csv(self):
        """Query S3 and write output to csv file."""
        s3_client = S3Client()
        with open(self.data_report_file_name, 'wb') as data_report_file:
            LOGGER.debug('Fetching enterprise course structure report from S3')
            s3_client.get_enterprise_report(self.get_s3_csv_path(), data_report_file)
        return [data_report_file]

    def _generate_enterprise_report_completion_csv(self):
        """Query S3 and write output to csv file."""
        s3_client = S3Client()
        with open(self.data_report_file_name, 'wb') as data_report_file:
            LOGGER.debug('Fetching enterprise completion report from S3')
            s3_client.get_enterprise_report(self.get_s3_csv_path(), data_report_file)
        return [data_report_file]

    def _generate_enterprise_report_progress_v2_csv(self):
        """Query the Enterprise Data API to get progress data to be turned into a CSV."""
        enrollments = EnterpriseDataApiClient().get_enterprise_enrollments(self.enterprise_customer_uuid)['results']
        if not enrollments:
            return []
        with open(self.data_report_file_name, 'w') as data_report_file:
            writer = csv.writer(data_report_file)
            writer.writerow(list(OrderedDict(sorted(enrollments[0].items())).keys()))
            for enrollment in enrollments:
                writer.writerow(list(OrderedDict(sorted(enrollment.items())).values()))
        return [data_report_file]

    def _generate_enterprise_report_progress_v3_csv(self):
        """Query the Enterprise Data V1 API to get progress data to be turned into a CSV."""
        enrollments = EnterpriseDataV1ApiClient().get_enterprise_enrollments(self.enterprise_customer_uuid)['results']
        if not enrollments:
            return []
        with open(self.data_report_file_name, 'w') as data_report_file:
            writer = csv.writer(data_report_file)
            csv_header = list(enrollments[0].keys())
            writer.writerow(csv_header)
            for enrollment in enrollments:
                writer.writerow(list(enrollment.values()))
        return [data_report_file]

    def _generate_enterprise_report_engagement_csv(self):
        """Query the Analytics Data API to get engagement data to be turned into a CSV."""
        engagements = AnalyticsDataApiClient().get_enterprise_engagements(self.enterprise_customer_uuid)['results']
        if not engagements:
            return []
        with open(self.data_report_file_name, 'w') as data_report_file:
            writer = csv.writer(data_report_file)
            writer.writerow(list(OrderedDict(sorted(engagements[0].items())).keys()))
            for engagement in engagements:
                writer.writerow(list(OrderedDict(sorted(engagement.items())).values()))
        return [data_report_file]

    def _generate_enterprise_report_progress_v2_json(self):
        """
        Query the Enterprise Data API to get progress data to be turned into json.
        """
        enrollments = EnterpriseDataApiClient().get_enterprise_enrollments(self.enterprise_customer_uuid)['results']
        report_data = enrollments if enrollments else []

        with open(self.data_report_file_name, 'w') as data_report_file:
            json.dump(report_data, data_report_file, indent=4)

        return [data_report_file]

    def _generate_enterprise_report_progress_v3_json(self):
        """
        Query the Enterprise Data V1 API to get progress data to be turned into json.
        """
        enrollments = EnterpriseDataV1ApiClient().get_enterprise_enrollments(self.enterprise_customer_uuid)['results']
        report_data = enrollments if enrollments else []

        with open(self.data_report_file_name, 'w') as data_report_file:
            json.dump(report_data, data_report_file, indent=4)

        return [data_report_file]

    def _generate_enterprise_report_catalog_csv(self):
        """
        Query the Enterprise Customer Catalog API and turn results into multiple CSV files.

        Note that at the current time, the CSV is unconventional. It is produced with multiple headers,
        one per each JSON object retrieved from the catalog API.
        """
        content_metadata = self.__get_content_metadata()
        LOGGER.debug('Grouping up content metadata by type...')
        grouped_content_metadata = {}
        for item in content_metadata:
            content_type = item['content_type']
            if content_type not in grouped_content_metadata:
                grouped_content_metadata[content_type] = [item]
            else:
                grouped_content_metadata[content_type].append(item)

        LOGGER.info('Beginning to write content metadata groups to CSVs...')
        files = []
        for content_type, grouped_items in list(grouped_content_metadata.items()):
            with open(self.data_report_file_name_with.format(content_type), 'w') as data_report_file:
                writer = csv.writer(data_report_file)
                if grouped_items:
                    # Write single row of headers in csv.
                    writer.writerow(generate_data(grouped_items[0], target='key'))

                for item in grouped_items:
                    writer.writerow(generate_data(item, target='value'))

                files.append(data_report_file)
        return files

    def _generate_enterprise_report_catalog_json(self):
        """Query the Enterprise Catalog API and transfer the results into a JSON file."""
        content_metadata = self.__get_content_metadata()
        with open(self.data_report_file_name, 'w') as data_report_file:
            json.dump(list(content_metadata), data_report_file, indent=4)
        return [data_report_file]

    def __get_content_metadata(self):
        """Get content metadata from the Enterprise Catalog API."""
        customer_catalogs = extract_catalog_uuids_from_reporting_config(self.reporting_config)
        enterprise_catalog_api_client = EnterpriseCatalogAPIClient()

        if not customer_catalogs.get('results'):
            customer_catalogs = enterprise_catalog_api_client.get_customer_catalogs(self.enterprise_customer_uuid)

        content_metadata = enterprise_catalog_api_client.get_content_metadata(customer_catalogs)
        return content_metadata



# Generated by Django 4.2.14 on 2024-08-08 07:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0040_auto_20240718_0536_squashed_0043_alter_enterpriselearnerenrollment_enterprise_enrollment_id'),
    ]

    operations = [
        migrations.CreateModel(
            name='EnterpriseExecEdLCModulePerformance',
            fields=[
                ('module_performance_unique_id', models.CharField(max_length=350, primary_key=True, serialize=False)),
                ('registration_id', models.PositiveIntegerField(null=True)),
                ('subsidy_transaction_id', models.UUIDField(null=True)),
                ('enterprise_customer_uuid', models.UUIDField(db_index=True, null=True)),
                ('ocm_lms_user_id', models.PositiveIntegerField(null=True)),
                ('is_internal_subsidy', models.BooleanField(null=True)),
                ('ocm_enrollment_id', models.PositiveIntegerField(null=True)),
                ('ocm_courserun_key', models.CharField(max_length=255, null=True)),
                ('university_name', models.CharField(max_length=500, null=True)),
                ('university_abbreviation', models.CharField(max_length=255, null=True)),
                ('partner_short_name', models.CharField(max_length=255, null=True)),
                ('university_country', models.CharField(max_length=255, null=True)),
                ('school', models.CharField(max_length=500, null=True)),
                ('faculty', models.CharField(max_length=500, null=True)),
                ('department', models.CharField(max_length=500, null=True)),
                ('course_code', models.PositiveIntegerField(null=True)),
                ('course_name', models.CharField(db_index=True, max_length=500)),
                ('course_abbreviation', models.CharField(max_length=128, null=True)),
                ('course_abbreviation_short', models.CharField(max_length=64, null=True)),
                ('course_type', models.CharField(max_length=128, null=True)),
                ('subject_vertical', models.CharField(max_length=128, null=True)),
                ('presentation_abbreviation', models.CharField(max_length=128, null=True)),
                ('presentation_name', models.CharField(max_length=500, null=True)),
                ('presentation_code', models.PositiveIntegerField(null=True)),
                ('presentation_close_date', models.DateField(null=True)),
                ('presentation_start_date', models.DateField(null=True)),
                ('promotion_code', models.CharField(max_length=255, null=True)),
                ('promotion_category_name', models.CharField(max_length=255, null=True)),
                ('product_type', models.CharField(max_length=128, null=True)),
                ('product_life_cycle_status', models.CharField(max_length=128, null=True)),
                ('enrolment_id', models.PositiveIntegerField(null=True)),
                ('olc_user_id', models.PositiveIntegerField(null=True)),
                ('first_name', models.CharField(max_length=255, null=True)),
                ('last_name', models.CharField(max_length=255, null=True)),
                ('username', models.CharField(db_index=True, max_length=255)),
                ('status', models.CharField(max_length=128, null=True)),
                ('company_name', models.CharField(max_length=255, null=True)),
                ('module_number', models.PositiveIntegerField(null=True)),
                ('module_name', models.CharField(max_length=255, null=True)),
                ('module_1_release_date', models.DateField(null=True)),
                ('last_module_release_date', models.DateField(null=True)),
                ('last_module_end_date', models.DateField(null=True)),
                ('final_mark', models.DecimalField(decimal_places=2, max_digits=38, null=True)),
                ('assign_grade', models.DecimalField(decimal_places=2, max_digits=38, null=True)),
                ('last_access', models.DateField(null=True)),
                ('all_activities_completed_count', models.PositiveIntegerField(null=True)),
                ('all_activities_total_count', models.PositiveIntegerField(null=True)),
                ('percentage_completed_activities', models.DecimalField(decimal_places=2, max_digits=38, null=True)),
                ('extensions_requested', models.PositiveIntegerField(null=True)),
                ('module_grade', models.DecimalField(decimal_places=2, max_digits=38, null=True)),
                ('log_viewed', models.PositiveIntegerField(null=True)),
                ('hours_online', models.DecimalField(decimal_places=2, max_digits=38, null=True)),
                ('orientation_module_accessed', models.CharField(max_length=128, null=True)),
                ('graded_activities_completed_count', models.PositiveIntegerField(null=True)),
                ('graded_activities_total_count', models.PositiveIntegerField(null=True)),
                ('percentage_completed_graded_activities', models.DecimalField(decimal_places=2, max_digits=38, null=True)),
                ('assessment_activities_completed_count', models.PositiveIntegerField(null=True)),
                ('assessment_activities_total_count', models.PositiveIntegerField(null=True)),
                ('course_material_activities_completed_count', models.PositiveIntegerField(null=True)),
                ('course_material_activities_total_count', models.PositiveIntegerField(null=True)),
                ('discussion_forum_activities_completed_count', models.PositiveIntegerField(null=True)),
                ('discussion_forum_activities_total_count', models.PositiveIntegerField(null=True)),
                ('pass_grade', models.DecimalField(decimal_places=2, max_digits=38, null=True)),
            ],
            options={
                'verbose_name': 'Exec Ed LC Module Performance',
                'verbose_name_plural': 'Exec Ed LC Module Performance',
                'db_table': 'exec_ed_lc_module_performance',
                'ordering': ['last_access'],
            },
        ),
    ]



"""
Module for interacting with the fact_enrollment_engagement_day_admin_dash table.
"""
from datetime import date
from uuid import UUID

from enterprise_data.cache.decorators import cache_it
from enterprise_data.utils import find_first

from ..queries import FactEngagementAdminDashQueries
from ..utils import run_query
from .base import BaseTable

NULL_EMAIL_TEXT = 'learners who have not shared consent'


class FactEngagementAdminDashTable(BaseTable):
    """
    Class for communicating with the fact_enrollment_engagement_day_admin_dash table.
    """
    queries = FactEngagementAdminDashQueries()

    @cache_it()
    def get_learning_hours_and_daily_sessions(self, enterprise_customer_uuid: UUID, start_date: date, end_date: date):
        """
        Get the learning hours and daily sessions for the given enterprise customer.

        Arguments:
            enterprise_customer_uuid (UUID): The UUID of the enterprise customer.
            start_date (date): The start date.
            end_date (date): The end date.

        Returns:
            (tuple<float, int>): The learning hours and daily sessions.
        """
        results = run_query(
            query=self.queries.get_learning_hours_and_daily_sessions_query(),
            params={
                'enterprise_customer_uuid': enterprise_customer_uuid,
                'start_date': start_date,
                'end_date': end_date,
            }
        )
        if not results:
            return 0.0, 0

        return tuple(results[0])

    @cache_it()
    def get_engagement_count(self, enterprise_customer_uuid: UUID, start_date: date, end_date: date):
        """
        Get the total number of engagements for the given enterprise customer.

        Arguments:
            enterprise_customer_uuid (UUID): The UUID of the enterprise customer.
            start_date (date): The start date.
            end_date (date): The end date.

        Returns:
            int: The total number of engagements.
        """
        results = run_query(
            query=self.queries.get_engagement_count_query(),
            params={
                'enterprise_customer_uuid': enterprise_customer_uuid,
                'start_date': start_date,
                'end_date': end_date,
            }
        )
        if not results:
            return 0
        return results[0][0]

    @cache_it()
    def get_all_engagements(
            self, enterprise_customer_uuid: UUID, start_date: date, end_date: date, limit: int, offset: int
    ):
        """
        Get all engagement data for the given enterprise customer.

        Arguments:
            enterprise_customer_uuid (UUID): The UUID of the enterprise customer.
            start_date (date): The start date.
            end_date (date): The end date.
            limit (int): The maximum number of records to return.
            offset (int): The number of records to skip.

        Returns:
            list<dict>: A list of dictionaries containing the engagement data.
        """
        return run_query(
            query=self.queries.get_all_engagement_query(),
            params={
                'enterprise_customer_uuid': enterprise_customer_uuid,
                'start_date': start_date,
                'end_date': end_date,
                'limit': limit,
                'offset': offset,
            },
            as_dict=True,
        )

    @cache_it()
    def get_top_courses_by_engagement(self, enterprise_customer_uuid: UUID, start_date: date, end_date: date):
        """
        Get the top courses by user engagement for the given enterprise customer.

        Arguments:
            enterprise_customer_uuid (UUID): The UUID of the enterprise customer.
            start_date (date): The start date.
            end_date (date): The end date.

        Returns:
            list<dict>: A list of dictionaries containing the course data.
        """
        return run_query(
            query=self.queries.get_top_courses_by_engagement_query(),
            params={
                'enterprise_customer_uuid': enterprise_customer_uuid,
                'start_date': start_date,
                'end_date': end_date,
            },
            as_dict=True,
        )

    @cache_it()
    def get_top_subjects_by_engagement(self, enterprise_customer_uuid: UUID, start_date: date, end_date: date):
        """
        Get the top subjects by user engagement for the given enterprise customer.

        Arguments:
            enterprise_customer_uuid (UUID): The UUID of the enterprise customer.
            start_date (date): The start date.
            end_date (date): The end date.

        Returns:
            list<dict>: A list of dictionaries containing the subject data.
        """
        return run_query(
            query=self.queries.get_top_subjects_by_engagement_query(),
            params={
                'enterprise_customer_uuid': enterprise_customer_uuid,
                'start_date': start_date,
                'end_date': end_date,
            },
            as_dict=True,
        )

    @cache_it()
    def get_engagement_time_series_data(self, enterprise_customer_uuid: UUID, start_date: date, end_date: date):
        """
        Get the engagement time series data.

        Arguments:
            enterprise_customer_uuid (UUID): The UUID of the enterprise customer.
            start_date (date): The start date.
            end_date (date): The end date.

        Returns:
            list<dict>: A list of dictionaries containing the engagement time series data.
        """
        return run_query(
            query=self.queries.get_engagement_time_series_data_query(),
            params={
                'enterprise_customer_uuid': enterprise_customer_uuid,
                'start_date': start_date,
                'end_date': end_date,
            },
            as_dict=True,
        )

    @cache_it()
    def _get_engagement_data_for_leaderboard(
            self,
            enterprise_customer_uuid: UUID,
            start_date: date,
            end_date: date,
            limit: int,
            offset: int,
            include_null_email: bool,

    ):
        """
        Get the engagement data for leaderboard.

        The engagement data would include fields like learning time, session length of
        the enterprise learners to show in the leaderboard.

        Arguments:
            enterprise_customer_uuid (UUID): The UUID of the enterprise customer.
            start_date (date): The start date.
            end_date (date): The end date.
            limit (int): The maximum number of records to return.
            offset (int): The number of records to skip.
            include_null_email (bool): If True, only fetch data for NULL emails.

        Returns:
            list[dict]: The engagement data for leaderboard.
        """
        engagements = run_query(
            query=self.queries.get_engagement_data_for_leaderboard_query(),
            params={
                'enterprise_customer_uuid': enterprise_customer_uuid,
                'start_date': start_date,
                'end_date': end_date,
                'limit': limit,
                'offset': offset,
            },
            as_dict=True,
        )

        if include_null_email:
            engagement_for_null_email = run_query(
                query=self.queries.get_engagement_data_for_leaderboard_null_email_only_query(),
                params={
                    'enterprise_customer_uuid': enterprise_customer_uuid,
                    'start_date': start_date,
                    'end_date': end_date,
                },
                as_dict=True,
            )
            engagements += engagement_for_null_email
        return engagements

    @cache_it()
    def _get_completion_data_for_leaderboard_query(
            self,
            enterprise_customer_uuid: UUID,
            start_date: date,
            end_date: date,
            email_list: list,
            include_null_email: bool,
    ):
        """
        Get the completion data for leaderboard.

        The completion data would include fields like course completion count of enterprise learners of the
        given enterprise customer.

        Arguments:
            enterprise_customer_uuid (UUID): The UUID of the enterprise customer.
            start_date (date): The start date.
            end_date (date): The end date.
            email_list (list<str>): List of emails of the enterprise learners.
            include_null_email (bool): If True, only fetch data for NULL emails.

        Returns:
            list[dict]: The engagement data for leaderboard.
        """

        completions = run_query(
            query=self.queries.get_completion_data_for_leaderboard_query(email_list),
            params={
                'enterprise_customer_uuid': enterprise_customer_uuid,
                'start_date': start_date,
                'end_date': end_date,
            },
            as_dict=True,
        )

        if include_null_email:
            completions_for_null_email = run_query(
                query=self.queries.get_completion_data_for_leaderboard_null_email_only_query(),
                params={
                    'enterprise_customer_uuid': enterprise_customer_uuid,
                    'start_date': start_date,
                    'end_date': end_date,
                },
                as_dict=True,
            )
            completions += completions_for_null_email

        return completions

    def get_all_leaderboard_data(
            self,
            enterprise_customer_uuid: UUID,
            start_date: date,
            end_date: date,
            limit: int,
            offset: int,
            total_count: int,
    ):
        """
        Get the leaderboard data for the given enterprise customer.

        Arguments:
            enterprise_customer_uuid (UUID): The UUID of the enterprise customer.
            start_date (date): The start date.
            end_date (date): The end date.
            limit (int): The maximum number of records to return.
            offset (int): The number of records to skip.
            total_count (int): The total number of records.

        Returns:
            list[dict]: The leaderboard data.
        """
        include_null_email = False
        # If this is the last or only page, we need to include NULL emails record.
        if total_count <= offset + limit:
            include_null_email = True

        engagement_data = self._get_engagement_data_for_leaderboard(
            enterprise_customer_uuid=enterprise_customer_uuid,
            start_date=start_date,
            end_date=end_date,
            limit=limit,
            offset=offset,
            include_null_email=include_null_email,
        )
        # If there is no data, no need to proceed.
        if not engagement_data:
            return []

        engagement_data_dict = {
            engagement['email']: engagement for engagement in engagement_data if engagement['email']
        }
        completion_data = self._get_completion_data_for_leaderboard_query(
            enterprise_customer_uuid=enterprise_customer_uuid,
            start_date=start_date,
            end_date=end_date,
            email_list=list(engagement_data_dict.keys()),
            include_null_email=include_null_email,
        )
        for completion in completion_data:
            email = completion['email']
            engagement_data_dict[email]['course_completion_count'] = completion['course_completion_count']

        if include_null_email:
            engagement_data_dict['None'] = find_first(engagement_data, lambda x: x['email'] is None) or {}
            completion = find_first(completion_data, lambda x: x['email'] is None) or \
                {'course_completion_count': ''}
            engagement_data_dict['None']['course_completion_count'] = completion['course_completion_count']
            engagement_data_dict['None']['email'] = NULL_EMAIL_TEXT

        return list(engagement_data_dict.values())

    @cache_it()
    def get_leaderboard_data_count(self, enterprise_customer_uuid: UUID, start_date: date, end_date: date):
        """
        Get the total number of leaderboard records for the given enterprise customer.

        Arguments:
            enterprise_customer_uuid (UUID): The UUID of the enterprise customer.
            start_date (date): The start date.
            end_date (date): The end date.

        Returns:
            (int): The total number of leaderboard records.
        """
        results = run_query(
            query=self.queries.get_leaderboard_data_count_query(),
            params={
                'enterprise_customer_uuid': enterprise_customer_uuid,
                'start_date': start_date,
                'end_date': end_date,
            }
        )
        if not results:
            return 0
        return results[0][0]



"""
Management command for creating enterprise learner.
"""


import sys

from django.core.management.base import BaseCommand, CommandError

import enterprise_data.tests.test_utils


class Command(BaseCommand):
    """
    Management command for creating `EnterpriseLearner` instances for dummy data for learner progress report (LPR) V1.
    """
    help = 'Creates an EnterpriseLearner with randomized data.'

    def add_arguments(self, parser):
        parser.add_argument(
            'enterprise_customer_uuid',
            type=str,
            help='UUID for an enterprise'
        )

    def handle(self, *args, **options):
        enterprise_customer_uuid = options['enterprise_customer_uuid']
        try:
            ent_user = enterprise_data.tests.test_utils.EnterpriseLearnerFactory(
                enterprise_customer_uuid=enterprise_customer_uuid
            )
        except Exception as exc:
            info = (
                'Error trying to create EnterpriseLearner with uuid '
                '{}: {}'.format(enterprise_customer_uuid, exc)
            )
            raise CommandError(info) from exc

        info = '\n\nCreated EnterpriseLearner with id {} for enterprise with uuid {}\n'.format(
            ent_user.enterprise_user_id,
            ent_user.enterprise_customer_uuid
        )
        sys.stdout.write(info)

        # TODO: Need to update this to the correct comment for LPR V1 once that is added.
        info = (
            'You can create some enrollments for this user by running the following '
            'command:\n\n    ./manage.py create_enterprise_learner_enrollment_lpr_v1 {} {}\n\n'.format(
                ent_user.enterprise_customer_uuid,
                ent_user.enterprise_user_id,
            )
        )
        sys.stdout.write(info)



# Generated by Django 2.2.17 on 2021-07-03 18:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0024_auto_20210602_1811'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='enterpriselearner',
            options={'ordering': ['-user_email'], 'verbose_name': 'Enterprise Learner', 'verbose_name_plural': 'Enterprise Learner'},
        ),
    ]



"""
Test factories.
"""


from datetime import datetime

import factory
import pytz
from faker import Factory as FakerFactory
from faker.providers import misc

from django.contrib.auth import get_user_model

from enterprise_data.models import (
    EnterpriseEnrollment,
    EnterpriseGroupMembership,
    EnterpriseLearner,
    EnterpriseLearnerEnrollment,
    EnterpriseOffer,
    EnterpriseSubsidyBudget,
    EnterpriseUser,
)

FAKER = FakerFactory.create()
FAKER.add_provider(misc)
User = get_user_model()


class EnterpriseEnrollmentFactory(factory.django.DjangoModelFactory):
    """
    EnterpriseCourseEnrollment factory.

    Creates an instance of EnterpriseCourseEnrollment with minimal boilerplate.
    """

    class Meta:
        """
        Meta for EnterpriseCourseEnrollmentFactory.
        """

        model = EnterpriseEnrollment

    id = factory.lazy_attribute(lambda x: FAKER.random_int(min=1, max=999999))  # pylint: disable=no-member,invalid-name
    enterprise_id = str(FAKER.uuid4())  # pylint: disable=no-member
    lms_user_id = factory.lazy_attribute(lambda x: FAKER.random_int(min=1))  # pylint: disable=no-member
    course_id = factory.lazy_attribute(lambda x: FAKER.slug())  # pylint: disable=no-member
    enrollment_created_timestamp = factory.lazy_attribute(lambda x: '2018-01-01T00:00:00Z')
    user_current_enrollment_mode = factory.lazy_attribute(lambda x: 'verified')
    has_passed = factory.lazy_attribute(lambda x: FAKER.boolean())  # pylint: disable=no-member
    consent_granted = True
    course_title = factory.lazy_attribute(lambda x: ' '.join(FAKER.words(nb=2)).title())  # pylint: disable=no-member
    course_start = factory.lazy_attribute(lambda x: FAKER.date_time_between(  # pylint: disable=no-member
        start_date='-2M',
        end_date='+2M',
        tzinfo=pytz.utc))
    user_email = factory.lazy_attribute(lambda x: FAKER.email())  # pylint: disable=no-member
    current_grade = factory.lazy_attribute(
        lambda x: FAKER.pyfloat(right_digits=2, min_value=0, max_value=1))  # pylint: disable=no-member

    @factory.lazy_attribute
    def course_end(self):
        return FAKER.date_time_between(   # pylint: disable=no-member
            start_date=self.course_start,
            end_date="+8M",
            tzinfo=pytz.utc)

    @factory.lazy_attribute
    def passed_timestamp(self):
        """ Create a passed timestamp if a course has been passed """
        if self.has_passed:
            return FAKER.date_time_between(   # pylint: disable=no-member
                start_date=self.course_start,
                end_date=self.course_end,
                tzinfo=pytz.utc)
        return None


class UserFactory(factory.django.DjangoModelFactory):
    """
    User Factory.

    Creates an instance of User with minimal boilerplate
    """
    class Meta:
        model = User
        django_get_or_create = ('email', 'username')

    _DEFAULT_PASSWORD = 'test'

    username = factory.Sequence('robot{}'.format)
    email = factory.Sequence('robot+test+{}@edx.org'.format)
    password = factory.PostGenerationMethodCall('set_password', _DEFAULT_PASSWORD)
    first_name = factory.Sequence('Robot{}'.format)
    last_name = 'Test'
    is_staff = factory.lazy_attribute(lambda x: False)
    is_active = True
    is_superuser = False
    last_login = datetime(2012, 1, 1, tzinfo=pytz.utc)
    date_joined = datetime(2011, 1, 1, tzinfo=pytz.utc)


class EnterpriseUserFactory(factory.django.DjangoModelFactory):
    """
    Enterprise User Factory.

    Creates an instance of Enterprise User with minimal boilerplate
    """
    class Meta:
        model = EnterpriseUser

    enterprise_id = str(FAKER.uuid4())  # pylint: disable=no-member
    lms_user_id = factory.lazy_attribute(lambda x: FAKER.random_int(min=1))  # pylint: disable=no-member
    enterprise_user_id = factory.lazy_attribute(lambda x: FAKER.random_int(min=1))  # pylint: disable=no-member
    enterprise_sso_uid = factory.lazy_attribute(lambda x: FAKER.text(max_nb_chars=255))  # pylint: disable=no-member
    user_account_creation_timestamp = datetime(2011, 1, 1, tzinfo=pytz.utc)
    user_username = factory.Sequence('robot{}'.format)
    user_email = factory.lazy_attribute(lambda x: FAKER.email())  # pylint: disable=no-member
    user_country_code = factory.lazy_attribute(lambda x: FAKER.country_code())  # pylint: disable=no-member
    last_activity_date = datetime(2012, 1, 1).date()


class EnterpriseLearnerFactory(factory.django.DjangoModelFactory):
    """
    EnterpriseLearner model Factory.
    """
    class Meta:
        model = EnterpriseLearner

    enterprise_user_id = factory.lazy_attribute(lambda x: FAKER.random_int(min=1))  # pylint: disable=no-member
    enterprise_customer_uuid = str(FAKER.uuid4())  # pylint: disable=no-member
    enterprise_user_created = FAKER.past_datetime(start_date='-30d')
    enterprise_user_modified = FAKER.past_datetime(start_date='-30d')
    enterprise_user_active = FAKER.pybool()
    lms_user_id = factory.lazy_attribute(
        lambda x: FAKER.random_int(min=1, max=999999)  # pylint: disable=no-member
    )
    is_linked = FAKER.pybool()
    user_username = factory.Sequence('robot{}'.format)
    user_email = factory.lazy_attribute(lambda x: FAKER.email())  # pylint: disable=no-member
    lms_user_created = FAKER.past_datetime(start_date='-60d')
    lms_last_login = FAKER.past_datetime(start_date='-10d')
    lms_user_country = factory.lazy_attribute(lambda x: FAKER.country_code())  # pylint: disable=no-member
    enterprise_sso_uid = factory.lazy_attribute(lambda x: FAKER.text(max_nb_chars=255))  # pylint: disable=no-member
    last_activity_date = datetime(2012, 1, 1).date()
    created_at = FAKER.past_datetime(start_date='-30d')


class EnterpriseLearnerEnrollmentFactory(factory.django.DjangoModelFactory):
    """
    EnterpriseLearnerEnrollment model factory.
    """

    class Meta:
        model = EnterpriseLearnerEnrollment

    enrollment_id = factory.lazy_attribute(
        lambda x: FAKER.random_int(min=1, max=999999)  # pylint: disable=no-member
    )
    enterprise_enrollment_id = factory.lazy_attribute(
        lambda x: FAKER.random_int(min=1, max=999999)  # pylint: disable=no-member
    )
    enterprise_customer_uuid = str(FAKER.uuid4())  # pylint: disable=no-member
    courserun_key = factory.lazy_attribute(lambda x: FAKER.slug())  # pylint: disable=no-member
    enrollment_date = factory.lazy_attribute(lambda x: '2018-01-01')
    user_current_enrollment_mode = factory.lazy_attribute(lambda x: 'verified')
    has_passed = factory.lazy_attribute(lambda x: FAKER.boolean())  # pylint: disable=no-member
    course_title = factory.lazy_attribute(lambda x: ' '.join(FAKER.words(nb=2)).title())  # pylint: disable=no-member
    course_start_date = factory.lazy_attribute(lambda x: FAKER.date_between(  # pylint: disable=no-member
        start_date='-2M',
        end_date='+2M',
    ))
    course_list_price = factory.lazy_attribute(
        lambda x: FAKER.pyfloat(min_value=25.0, max_value=1000.0, right_digits=2)
    )
    current_grade = factory.lazy_attribute(
        lambda x: FAKER.pyfloat(right_digits=2, min_value=0, max_value=1)  # pylint: disable=no-member
    )
    letter_grade = factory.lazy_attribute(lambda x: ' '.join(FAKER.words(nb=2)).title())
    progress_status = factory.lazy_attribute(lambda x: ' '.join(FAKER.words(nb=2)).title())
    enterprise_user_id = factory.Sequence(lambda n: n)
    user_email = factory.lazy_attribute(lambda x: FAKER.email())  # pylint: disable=no-member
    user_username = factory.Sequence('robot{}'.format)
    user_first_name = factory.Sequence('Robot First {}'.format)
    user_last_name = factory.Sequence('Robot Last {}'.format)
    user_account_creation_date = factory.lazy_attribute(lambda x: '2018-01-01')
    user_country_code = factory.lazy_attribute(lambda x: FAKER.country_code())
    is_subsidy = factory.lazy_attribute(lambda x: FAKER.boolean())  # pylint: disable=no-member
    course_product_line = factory.LazyAttribute(lambda x: FAKER.pystr())

    @factory.lazy_attribute
    def course_end_date(self):
        return FAKER.date_between(   # pylint: disable=no-member
            start_date=self.course_start_date,
            end_date="+8M"
        )

    @factory.lazy_attribute
    def passed_date(self):
        """ Create a passed timestamp if a course has been passed """
        if self.has_passed and self.is_consent_granted:
            return FAKER.date_between(  # pylint: disable=no-member
                start_date=self.course_start_date,
                end_date=self.course_end_date,
            )
        return None

    @factory.lazy_attribute
    def last_activity_date(self):
        """ Create a date in between course start and end timestamp"""
        if self.is_consent_granted:
            return FAKER.date_between(  # pylint: disable=no-member
                start_date=self.course_start_date,
                end_date=self.course_end_date,
            )
        return None

    @factory.post_generation
    def set_fields_according_to_consent(
            obj,
            create,
            extracted,
            **kwargs
    ):  # pylint: disable=unused-argument, missing-function-docstring
        dsc_dependent_fields = [
            'last_activity_date',
            'progress_status',
            'passed_date',
            'current_grade',
            'letter_grade',
            'enterprise_user_id',
            'user_email',
            'user_account_creation_date',
            'user_country_code',
            'user_username',
            'user_first_name',
            'user_last_name',
        ]
        if create and not obj.is_consent_granted:
            for field in dsc_dependent_fields:
                setattr(obj, field, None)
            obj.save()


class EnterpriseSubsidyBudgetFactory(factory.django.DjangoModelFactory):
    """
    EnterpriseSubsidyBudget model Factory.
    """
    class Meta:
        model = EnterpriseSubsidyBudget

    id = factory.LazyAttribute(lambda x: FAKER.uuid4().replace('-', ''))
    subsidy_access_policy_uuid = factory.LazyAttribute(lambda x: FAKER.uuid4().replace('-', ''))
    subsidy_uuid = factory.LazyAttribute(lambda x: FAKER.uuid4().replace('-', ''))
    enterprise_customer_uuid = factory.LazyAttribute(lambda x: FAKER.uuid4().replace('-', ''))
    enterprise_customer_name = factory.LazyAttribute(lambda x: ' '.join(FAKER.words(nb=2)).title())
    subsidy_access_policy_description = factory.LazyAttribute(lambda x: ' '.join(FAKER.words(nb=2)).title())
    subsidy_title = factory.LazyAttribute(lambda x: ' '.join(FAKER.words(nb=2)).title())
    catalog_name = factory.LazyAttribute(lambda x: ' '.join(FAKER.words(nb=2)).title())
    subsidy_access_policy_type = 'PerLearnerSpendCreditAccessPolicy'
    date_created = factory.LazyAttribute(lambda _: datetime(2020, 1, 1, tzinfo=pytz.utc))
    subsidy_start_datetime = factory.LazyAttribute(lambda _: datetime(2022, 1, 1, tzinfo=pytz.utc))
    subsidy_expiration_datetime = factory.LazyAttribute(lambda _: datetime(2090, 1, 1, tzinfo=pytz.utc))
    is_active = True
    is_test = False
    ocm_usage = factory.lazy_attribute(
        lambda x: round(x.amount_of_policy_spent / 2, 2)
    )
    exec_ed_usage = factory.lazy_attribute(
        lambda x: round(x.amount_of_policy_spent / 2, 2)
    )
    usage_total = factory.lazy_attribute(
        lambda x: x.amount_of_policy_spent
    )
    enterprise_contract_discount_percent = factory.LazyAttribute(
        lambda _: FAKER.pyfloat(right_digits=2, min_value=1, max_value=100)
    )
    starting_balance = factory.LazyAttribute(
        lambda x: FAKER.pyfloat(right_digits=2, min_value=10000, max_value=100000)
    )
    amount_of_policy_spent = factory.lazy_attribute(
        lambda x: FAKER.pyfloat(right_digits=2, min_value=1, max_value=x.starting_balance)
    )
    remaining_balance = factory.lazy_attribute(
        lambda x: x.starting_balance - x.amount_of_policy_spent
    )
    percent_of_policy_spent = factory.lazy_attribute(
        lambda x: round(x.amount_of_policy_spent / x.starting_balance, 2)
    )


class EnterpriseGroupMembershipFactory(factory.django.DjangoModelFactory):
    """
    EnterpriseGroupMembership model Factory.
    """
    class Meta:
        model = EnterpriseGroupMembership

    enterprise_customer_id = factory.LazyAttribute(lambda _: FAKER.uuid4())
    enterprise_group_name = factory.LazyAttribute(lambda _: ' '.join(FAKER.words(nb=2)).title())
    enterprise_group_uuid = factory.LazyAttribute(lambda _: FAKER.uuid4())
    group_is_removed = factory.LazyAttribute(lambda _: FAKER.boolean())
    group_type = factory.LazyAttribute(lambda _: 'budget')
    activated_at = factory.LazyAttribute(lambda _: FAKER.date_time_this_decade(tzinfo=pytz.UTC))
    enterprise_customer_user_id = factory.LazyAttribute(lambda _: FAKER.random_int(min=1, max=10000))
    membership_is_removed = factory.LazyAttribute(lambda _: FAKER.boolean())
    membership_status = factory.LazyAttribute(lambda _: FAKER.word())
    enterprise_group_membership_uuid = factory.LazyAttribute(lambda _: FAKER.uuid4())

    is_applies_to_all_contexts = factory.LazyAttribute(lambda _: FAKER.boolean())


class EnterpriseOfferFactory(factory.django.DjangoModelFactory):
    """
    EnterpriseLearner model Factory.
    """
    class Meta:
        model = EnterpriseOffer

    offer_id = factory.LazyAttribute(lambda x: FAKER.pystr(min_chars=36, max_chars=200))
    enterprise_customer_uuid = factory.LazyAttribute(lambda x: FAKER.uuid4().replace('-', ''))
    enterprise_name = factory.LazyAttribute(lambda x: ' '.join(FAKER.words(nb=2)).title())
    max_discount = factory.LazyAttribute(lambda x: FAKER.pyfloat(right_digits=2, min_value=10000, max_value=100000))
    sum_course_price = factory.lazy_attribute(
        lambda x: FAKER.pyfloat(right_digits=2, min_value=1, max_value=x.max_discount)
    )
    status = 'Open'
    offer_type = 'Site'
    date_created = factory.LazyAttribute(lambda _: datetime(2012, 1, 1, tzinfo=pytz.utc))
    discount_type = 'Percentage'
    discount_value = factory.LazyAttribute(lambda _: FAKER.pyfloat(right_digits=2, min_value=1, max_value=100))
    total_discount_ecommerce = factory.lazy_attribute(
        lambda x: round(x.sum_course_price * (x.discount_value / 100), 2)
    )
    amount_of_offer_spent = factory.lazy_attribute(
        lambda x: x.total_discount_ecommerce
    )
    sum_amount_learner_paid = factory.lazy_attribute(
        lambda x: x.sum_course_price - x.amount_of_offer_spent
    )
    percent_of_offer_spent = factory.lazy_attribute(
        lambda x: round(x.amount_of_offer_spent / x.max_discount, 2)
    )
    remaining_balance = factory.lazy_attribute(
        lambda x: round(x.max_discount - x.amount_of_offer_spent, 2)
    )
    amount_offer_spent_ocm = factory.lazy_attribute(
        lambda x: round(x.amount_of_offer_spent / 2, 2)
    )
    amount_offer_spent_exec_ed = factory.lazy_attribute(
        lambda x: round(x.amount_of_offer_spent / 2, 2)
    )


def get_dummy_enterprise_api_data(**kwargs):
    """
    DRY method to get enterprise dummy data.

    Get dummy data for an enterprise from `enterprise-customer` API.
    """
    enterprise_api_dummy_data = {
        'uuid': kwargs.get('enterprise_id', 'ee5e6b3a-069a-4947-bb8d-d2dbc323396c'),
        'name': 'Enterprise ABC',
        'slug': 'enterprise_abc',
        'active': True,
        'enable_data_sharing_consent': kwargs.get('enable_data_sharing_consent', True),
        'enforce_data_sharing_consent': kwargs.get('enforce_data_sharing_consent', 'at_enrollment'),
        'branding_configuration': {},
        'identity_provider': 'saml-ubc',
        'enable_audit_data_reporting': kwargs.get('enable_audit_data_reporting', False),
        'replace_sensitive_sso_username': False
    }
    return enterprise_api_dummy_data


def get_dummy_engagements_data(enterprise_uuid: str, count=10):
    """
    Utility method to get dummy enrollment's data.
    """
    return [
        {
            'user_id': FAKER.random_int(min=1),
            'email': FAKER.email(),
            'enterprise_customer_uuid': enterprise_uuid,
            'course_key': FAKER.slug(),
            'enroll_type': 'verified',
            'activity_date': FAKER.date_time_between(
                start_date='-2M',
                end_date='+2M',
            ),
            'course_title': ' '.join(FAKER.words(nb=5)).title(),
            'course_subject': ' '.join(FAKER.words(nb=2)).title(),
            'is_engaged': FAKER.boolean(),
            'is_engaged_video': FAKER.boolean(),
            'is_engaged_forum': FAKER.boolean(),
            'is_engaged_problem': FAKER.boolean(),
            'is_active': FAKER.boolean(),
            'learning_time_seconds': FAKER.random_int(min=1),
        } for _ in range(count)
    ]


def get_dummy_enrollments_data(enterprise_uuid: str, count=10):
    """
    Utility method to get dummy enrollment's data.
    """
    return [
        {
            'enterprise_customer_name': ' '.join(FAKER.words(nb=2)).title(),
            'enterprise_customer_uuid': enterprise_uuid,
            'lms_enrollment_id': FAKER.random_int(min=1),
            'user_id': FAKER.random_int(min=1),
            'email': FAKER.email(),
            'course_key': FAKER.slug(),
            'courserun_key': FAKER.slug(),
            'course_id': FAKER.slug(),
            'course_subject': ' '.join(FAKER.words(nb=2)).title(),
            'course_title': ' '.join(FAKER.words(nb=5)).title(),
            'enterprise_enrollment_date': FAKER.date_time_between(
                start_date='-2M',
                end_date='+2M',
            ),
            'lms_enrollment_mode': 'verified',
            'enroll_type': 'verified',
            'program_title': ' '.join(FAKER.words(nb=2)).title(),
            'date_certificate_awarded': FAKER.date_time_between(
                start_date='-2M',
                end_date='+2M',
            ),
            'grade_percent': FAKER.pyfloat(right_digits=2, min_value=0, max_value=1),
            'cert_awarded': FAKER.boolean(),
            'date_certificate_created_raw': FAKER.date_time_between(
                start_date='-2M',
                end_date='+2M',
            ),
            'passed_date_raw': FAKER.date_time_between(
                start_date='-2M',
                end_date='+2M',
            ),
            'passed_date': FAKER.date_time_between(
                start_date='-2M',
                end_date='+2M',
            ),
            'has_passed': FAKER.boolean(),
        } for _ in range(count)
    ]


def get_dummy_skills_data(enterprise_uuid: str, count=10):
    """
    Utility method to get dummy skills data.
    """
    return [
        {
            'course_number': FAKER.random_int(min=1),
            'skill_type': 'skill_type',
            'skill_name': ' '.join(FAKER.words(nb=2)).title(),
            'skill_url': FAKER.url(),
            'confidence': FAKER.random_int(min=1),
            'skill_rank': FAKER.random_int(min=1),
            'course_title': ' '.join(FAKER.words(nb=5)).title(),
            'course_key': FAKER.slug(),
            'level_type': 'level_type',
            'primary_subject_name': ' '.join(FAKER.words(nb=2)).title(),
            'date': FAKER.date_time_between(
                start_date='-2M',
                end_date='+2M',
            ),
            'enterprise_customer_uuid': enterprise_uuid,
            'enterprise_customer_name': ' '.join(FAKER.words(nb=2)).title(),
            'enrolls': FAKER.random_int(min=1),
            'completions': FAKER.random_int(min=1),
        } for _ in range(count)
    ]



"""
Tests for enterprise completions analytics.
"""
from datetime import datetime

import ddt
from mock import patch
from rest_framework import status
from rest_framework.reverse import reverse
from rest_framework.test import APITransactionTestCase

from enterprise_data.admin_analytics.constants import ResponseType
from enterprise_data.tests.admin_analytics.mock_analytics_data import ENROLLMENTS
from enterprise_data.tests.mixins import JWTTestMixin
from enterprise_data.tests.test_utils import UserFactory
from enterprise_data_roles.constants import ENTERPRISE_DATA_ADMIN_ROLE
from enterprise_data_roles.models import EnterpriseDataFeatureRole, EnterpriseDataRoleAssignment


@ddt.ddt
class TestCompletionsStatsAPI(JWTTestMixin, APITransactionTestCase):
    """
    Tests for validating enterprise completions stats endpoint.
    """

    def setUp(self):
        """
        Setup method.
        """
        super().setUp()
        self.user = UserFactory(is_staff=True)
        role, __ = EnterpriseDataFeatureRole.objects.get_or_create(
            name=ENTERPRISE_DATA_ADMIN_ROLE
        )
        self.role_assignment = EnterpriseDataRoleAssignment.objects.create(
            role=role, user=self.user
        )
        self.client.force_authenticate(user=self.user)

        self.enterprise_id = 'ee5e6b3a-069a-4947-bb8d-d2dbc323396c'
        self.set_jwt_cookie()

        self.url = reverse(
            'v1:enterprise-admin-analytics-completions-stats',
            kwargs={'enterprise_uuid': self.enterprise_id},
        )

        get_enrollment_date_range_patcher = patch(
            'enterprise_data.api.v1.views.analytics_enrollments.FactEnrollmentAdminDashTable.get_enrollment_date_range',
            return_value=(datetime.now(), datetime.now())
        )

        get_enrollment_date_range_patcher.start()
        self.addCleanup(get_enrollment_date_range_patcher.stop)

    @patch(
        'enterprise_data.api.v1.views.analytics_enrollments.FactEnrollmentAdminDashTable.'
        'get_top_subjects_by_completions'
    )
    @patch(
        'enterprise_data.api.v1.views.analytics_enrollments.FactEnrollmentAdminDashTable.get_top_courses_by_completions'
    )
    @patch(
        'enterprise_data.api.v1.views.analytics_enrollments.FactEnrollmentAdminDashTable.'
        'get_completions_time_series_data'
    )
    def test_get(
            self,
            mock_get_completions_time_series_data,
            mock_get_top_courses_by_completions,
            mock_get_top_subjects_by_completions
    ):
        """
        Test the GET method to fetch charts data for enterprise completion works correctly.
        """
        mock_get_completions_time_series_data.return_value = []
        mock_get_top_courses_by_completions.return_value = []
        mock_get_top_subjects_by_completions.return_value = []

        params = {
            'start_date': '2020-01-01',
            'end_date': '2025-08-09',
        }
        response = self.client.get(self.url, params)
        assert response.status_code == status.HTTP_200_OK
        data = response.json()
        assert 'completions_over_time' in data
        assert 'top_courses_by_completions' in data
        assert 'top_subjects_by_completions' in data


@ddt.ddt
class TestCompletionsAPI(JWTTestMixin, APITransactionTestCase):
    """
    Tests for validating list endpoint of enterprise completions.
    """

    def setUp(self):
        """
        Setup method.
        """
        super().setUp()
        self.user = UserFactory(is_staff=True)
        role, __ = EnterpriseDataFeatureRole.objects.get_or_create(
            name=ENTERPRISE_DATA_ADMIN_ROLE
        )
        self.role_assignment = EnterpriseDataRoleAssignment.objects.create(
            role=role, user=self.user
        )
        self.client.force_authenticate(user=self.user)

        self.enterprise_id = 'ee5e6b3a-069a-4947-bb8d-d2dbc323396c'
        self.set_jwt_cookie()

        self.url = reverse(
            'v1:enterprise-admin-analytics-completions',
            kwargs={'enterprise_uuid': self.enterprise_id},
        )

        get_enrollment_date_range_patcher = patch(
            'enterprise_data.api.v1.views.analytics_enrollments.FactEnrollmentAdminDashTable.get_enrollment_date_range',
            return_value=(datetime.now(), datetime.now())
        )

        get_enrollment_date_range_patcher.start()
        self.addCleanup(get_enrollment_date_range_patcher.stop)

    @patch('enterprise_data.api.v1.views.analytics_enrollments.FactEnrollmentAdminDashTable.get_completion_count')
    @patch('enterprise_data.api.v1.views.analytics_enrollments.FactEnrollmentAdminDashTable.get_all_completions')
    def test_get(self, mock_get_all_completions, mock_get_completion_count):
        """
        Test the GET method for fetching enterprise completions works correctly.
        """
        mock_get_all_completions.return_value = ENROLLMENTS
        mock_get_completion_count.return_value = len(ENROLLMENTS)

        response = self.client.get(self.url + '?page=1&page_size=2')
        assert response.status_code == status.HTTP_200_OK
        data = response.json()
        assert data["next"] == f"http://testserver{self.url}?page=2&page_size=2"
        assert data["previous"] is None
        assert data["current_page"] == 1
        assert data["num_pages"] == 3
        assert data["count"] == 5

    @patch('enterprise_data.api.v1.views.analytics_enrollments.FactEnrollmentAdminDashTable.get_completion_count')
    @patch('enterprise_data.api.v1.views.analytics_enrollments.FactEnrollmentAdminDashTable.get_all_completions')
    def test_get_csv(self, mock_get_all_completions, mock_get_completion_count):
        """
        Test the GET method for the AdvanceAnalyticsIndividualEnrollmentsView return correct CSV data.
        """
        mock_get_all_completions.return_value = ENROLLMENTS
        mock_get_completion_count.return_value = len(ENROLLMENTS)
        response = self.client.get(self.url, {"response_type": ResponseType.CSV.value})
        assert response.status_code == status.HTTP_200_OK

        # verify the response headers
        assert response["Content-Type"] == "text/csv"

        # verify the response content
        content = b"".join(response.streaming_content).decode().splitlines()
        assert len(content) == 6

        # Verify CSV header.
        assert 'email,course_title,course_subject,enroll_type,passed_date' == content[0]

        # verify the content
        assert (
            'rebeccanelson@example.com,Re-engineered tangible approach,business-management,certificate,2021-08-25'
            in content
        )
        assert (
            'taylorjames@example.com,Re-engineered tangible approach,business-management,certificate,2021-09-01'
            in content
        )
        assert (
            'ssmith@example.com,Secured static capability,medicine,certificate,'
            in content
        )
        assert (
            'kathleenmartin@example.com,Horizontal solution-oriented hub,social-sciences,certificate,'
            in content
        )
        assert (
            'amber79@example.com,Streamlined zero-defect attitude,communication,certificate,'
            in content
        )



"""Mock data for enrollments"""

ENROLLMENTS = [
    {
        "enterprise_customer_name": "Hill Ltd",
        "enterprise_customer_uuid": "33ce656295e04ecfa2a77d407eb96f69",
        "lms_enrollment_id": 1013,
        "user_id": 8907,
        "email": "rebeccanelson@example.com",
        "course_key": "hEmW+tvk03",
        "courserun_key": "course-v1:hEmW+tvk03+1T9889",
        "course_id": "1681",
        "course_subject": "business-management",
        "course_title": "Re-engineered tangible approach",
        "enterprise_enrollment_date": "2021-07-04",
        "lms_enrollment_mode": "verified",
        "enroll_type": "certificate",
        "program_title": "Non-Program",
        "date_certificate_awarded": "2021-08-25",
        "grade_percent": 0.99,
        "cert_awarded": 1,
        "date_certificate_created_raw": "2021-08-25",
        "passed_date_raw": "2021-08-25",
        "passed_date": "2021-08-25",
        "has_passed": 1,
    },
    {
        "enterprise_customer_name": "Hill Ltd",
        "enterprise_customer_uuid": "33ce656295e04ecfa2a77d407eb96f69",
        "lms_enrollment_id": 9172,
        "user_id": 8369,
        "email": "taylorjames@example.com",
        "course_key": "hEmW+tvk03",
        "courserun_key": "course-v1:hEmW+tvk03+1T9889",
        "course_id": "1681",
        "course_subject": "business-management",
        "course_title": "Re-engineered tangible approach",
        "enterprise_enrollment_date": "2021-07-03",
        "lms_enrollment_mode": "verified",
        "enroll_type": "certificate",
        "program_title": "Non-Program",
        "date_certificate_awarded": "2021-09-01",
        "grade_percent": 0.93,
        "cert_awarded": 1,
        "date_certificate_created_raw": "2021-09-01",
        "passed_date_raw": "2021-09-01",
        "passed_date": "2021-09-01",
        "has_passed": 1,
    },
    {
        "enterprise_customer_name": "Hill Ltd",
        "enterprise_customer_uuid": "33ce656295e04ecfa2a77d407eb96f69",
        "lms_enrollment_id": 9552,
        "user_id": 8719,
        "email": "ssmith@example.com",
        "course_key": "qZJC+KFX86",
        "courserun_key": "course-v1:qZJC+KFX86+1T8918",
        "course_id": "1725",
        "course_subject": "medicine",
        "course_title": "Secured static capability",
        "enterprise_enrollment_date": "2021-05-11",
        "lms_enrollment_mode": "verified",
        "enroll_type": "certificate",
        "program_title": "Non-Program",
        "date_certificate_awarded": None,
        "grade_percent": 0.0,
        "cert_awarded": 0,
        "date_certificate_created_raw": None,
        "passed_date_raw": None,
        "passed_date": '2022-08-24',
        "has_passed": 0,
    },
    {
        "enterprise_customer_name": "Hill Ltd",
        "enterprise_customer_uuid": "33ce656295e04ecfa2a77d407eb96f69",
        "lms_enrollment_id": 3436,
        "user_id": 3125,
        "email": "kathleenmartin@example.com",
        "course_key": "QWXx+Jqz64",
        "courserun_key": "course-v1:QWXx+Jqz64+1T9449",
        "course_id": "4878",
        "course_subject": "social-sciences",
        "course_title": "Horizontal solution-oriented hub",
        "enterprise_enrollment_date": "2020-04-03",
        "lms_enrollment_mode": "verified",
        "enroll_type": "certificate",
        "program_title": "Non-Program",
        "date_certificate_awarded": None,
        "grade_percent": 0.0,
        "cert_awarded": 0,
        "date_certificate_created_raw": None,
        "passed_date_raw": None,
        "passed_date": "2022-08-24",
        "has_passed": 0,
    },
    {
        "enterprise_customer_name": "Hill Ltd",
        "enterprise_customer_uuid": "33ce656295e04ecfa2a77d407eb96f69",
        "lms_enrollment_id": 5934,
        "user_id": 4853,
        "email": "amber79@example.com",
        "course_key": "NOGk+UVD31",
        "courserun_key": "course-v1:NOGk+UVD31+1T4956",
        "course_id": "4141",
        "course_subject": "communication",
        "course_title": "Streamlined zero-defect attitude",
        "enterprise_enrollment_date": "2020-04-08",
        "lms_enrollment_mode": "verified",
        "enroll_type": "certificate",
        "program_title": "Non-Program",
        "date_certificate_awarded": None,
        "grade_percent": 0.0,
        "cert_awarded": 0,
        "date_certificate_created_raw": None,
        "passed_date_raw": None,
        "passed_date": "2022-08-20",
        "has_passed": 0,
    },
]



# Generated by Django 3.2.20 on 2023-09-07 11:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise_data', '0034_auto_20230907_0834'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisesubsidybudget',
            name='id',
            field=models.CharField(db_index=True, help_text='Hashed surrogate key based on subsidy_access_policy_uuid and subsidy_uuid', max_length=32, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='enterprisesubsidybudget',
            name='subsidy_access_policy_uuid',
            field=models.UUIDField(help_text='Budget Id'),
        ),
    ]
